@@ -8,16 +8,14 @@ from datetime import datetime, timedelta
 from functools import lru_cache, wraps
 from io import BytesIO
 from pathlib import Path
-from typing import Any, Callable, Iterable, List, Optional, Tuple, Union
-from urllib.parse import quote
+from typing import Any, Callable, Iterable, List, NoReturn, Optional, Tuple, Union
 from zlib import adler32
 
-from werkzeug.exceptions import NotFound
-from werkzeug.routing import BuildError
+from werkzeug.exceptions import abort as werkzeug_abort, NotFound
+from werkzeug.utils import redirect as werkzeug_redirect, safe_join
 from werkzeug.wrappers import Response as WerkzeugResponse
 
-from .ctx import _app_ctx_stack, _request_ctx_stack, _websocket_ctx_stack
-from .globals import current_app, request, session, websocket
+from .globals import _cv_request, current_app, request, request_ctx, session
 from .signals import message_flashed
 from .typing import FilePath
 from .utils import file_path_to_path
@@ -42,6 +40,20 @@ def get_debug_flag() -> bool:
     return value.lower() not in {"0", "false", "no"}
 
 
+def get_load_dotenv(default: bool = True) -> bool:
+    """Get whether the user has disabled loading default dotenv files by
+    setting :envvar:`QUART_SKIP_DOTENV`. The default is ``True``, load
+    the files.
+    :param default: What to return if the env var isn't set.
+    """
+    val = os.environ.get("QUART_SKIP_DOTENV")
+
+    if not val:
+        return default
+
+    return val.lower() in ("0", "false", "no")
+
+
 def get_env(default: Optional[str] = "production") -> str:
     """Reads QUART_ENV environment variable to determine in which environment
     the app is running on. Defaults to 'production' when unset.
@@ -126,11 +138,10 @@ def get_flashed_messages(
     all messages will be popped, but only those matching the filter
     returned. See :func:`~quart.helpers.flash` for message creation.
     """
-    flashes = _request_ctx_stack.top.flashes
+    flashes = request_ctx.flashes
     if flashes is None:
-        _request_ctx_stack.top.flashes = flashes = (
-            session.pop("_flashes") if "_flashes" in session else []
-        )
+        flashes = session.pop("_flashes") if "_flashes" in session else []
+        request_ctx.flashes = flashes
     if category_filter:
         flashes = [flash for flash in flashes if flash[0] in category_filter]
     if not with_categories:
@@ -138,6 +149,20 @@ def get_flashed_messages(
     return flashes
 
 
+def get_root_path(import_name: str) -> str:
+    """Find the root path of the *import_name*"""
+    module = sys.modules.get(import_name)
+    if module is not None and hasattr(module, "__file__"):
+        file_path = module.__file__
+    else:
+        loader = pkgutil.get_loader(import_name)
+        if loader is None or import_name == "__main__":
+            return str(Path.cwd())
+        else:
+            file_path = loader.get_filename(import_name)  # type: ignore
+    return str(Path(file_path).resolve().parent)
+
+
 def get_template_attribute(template_name: str, attribute: str) -> Any:
     """Load a attribute from a template.
 
@@ -175,61 +200,14 @@ def url_for(
         values: The values to build into the URL, as specified in
             the endpoint rule.
     """
-    app_context = _app_ctx_stack.top
-    request_context = _request_ctx_stack.top
-    websocket_context = _websocket_ctx_stack.top
-
-    if request_context is not None:
-        url_adapter = request_context.url_adapter
-        if endpoint.startswith("."):
-            if request.blueprint is not None:
-                endpoint = request.blueprint + endpoint
-            else:
-                endpoint = endpoint[1:]
-        if _external is None:
-            _external = False
-    elif websocket_context is not None:
-        url_adapter = websocket_context.url_adapter
-        if endpoint.startswith("."):
-            if websocket.blueprint is not None:
-                endpoint = websocket.blueprint + endpoint
-            else:
-                endpoint = endpoint[1:]
-        if _external is None:
-            _external = False
-    elif app_context is not None:
-        url_adapter = app_context.url_adapter
-        if _external is None:
-            _external = True
-    else:
-        raise RuntimeError("Cannot create a url outside of an application context")
-
-    if url_adapter is None:
-        raise RuntimeError(
-            "Unable to create a url adapter, try setting the SERVER_NAME config variable."
-        )
-    if _scheme is not None and not _external:
-        raise ValueError("External must be True for scheme usage")
-
-    app_context.app.inject_url_defaults(endpoint, values)
-
-    old_scheme = None
-    if _scheme is not None:
-        old_scheme = url_adapter.url_scheme
-        url_adapter.url_scheme = _scheme
-
-    try:
-        url = url_adapter.build(endpoint, values, method=_method, force_external=_external)
-    except BuildError as error:
-        return app_context.app.handle_url_build_error(error, endpoint, values)
-    finally:
-        if old_scheme is not None:
-            url_adapter.url_scheme = old_scheme
-
-    if _anchor is not None:
-        quoted_anchor = quote(_anchor)
-        url = f"{url}#{quoted_anchor}"
-    return url
+    return current_app.url_for(
+        endpoint,
+        _anchor=_anchor,
+        _method=_method,
+        _scheme=_scheme,
+        _external=_external,
+        **values,
+    )
 
 
 def stream_with_context(func: Callable) -> Callable:
@@ -251,7 +229,7 @@ def stream_with_context(func: Callable) -> Callable:
             return generator()
 
     """
-    request_context = _request_ctx_stack.top.copy()
+    request_context = _cv_request.get().copy()
 
     @wraps(func)
     async def generator(*args: Any, **kwargs: Any) -> Any:
@@ -288,25 +266,6 @@ def find_package(name: str) -> Tuple[Optional[Path], Path]:
         return sys_prefix, package_path
 
 
-def safe_join(directory: FilePath, *paths: FilePath) -> Path:
-    """Safely join the paths to the known directory to return a full path.
-
-    Raises:
-        NotFound: if the full path does not share a commonprefix with
-        the directory.
-    """
-    try:
-        safe_path = file_path_to_path(directory).resolve(strict=True)
-        full_path = file_path_to_path(directory, *paths).resolve(strict=True)
-    except FileNotFoundError:
-        raise NotFound()
-    try:
-        full_path.relative_to(safe_path)
-    except ValueError:
-        raise NotFound()
-    return full_path
-
-
 async def send_from_directory(
     directory: FilePath,
     file_name: str,
@@ -329,7 +288,7 @@ async def send_from_directory(
 
     See :func:`send_file` for the other arguments.
     """
-    file_path = safe_join(directory, file_name)
+    file_path = Path(safe_join(str(directory), file_name))
     if not file_path.is_file():
         raise NotFound()
     return await send_file(
@@ -425,3 +384,19 @@ def _split_blueprint_path(name: str) -> List[str]:
     while "." in bps[-1]:
         bps.append(bps[-1].rpartition(".")[0])
     return bps
+
+
+def abort(code: int, *args: Any, **kwargs: Any) -> NoReturn:  # type: ignore[misc]
+    """Raise an HTTPException for the given status code."""
+    if current_app:
+        current_app.aborter(code, *args, **kwargs)
+
+    werkzeug_abort(code, *args, **kwargs)
+
+
+def redirect(location: str, code: int = 302) -> WerkzeugResponse:
+    """Redirect to the location with the status code."""
+    if current_app:
+        return current_app.redirect(location, code=code)
+
+    return werkzeug_redirect(location, code=code)
