@@ -14,6 +14,7 @@ from typing import (
     Dict,
     Generator,
     List,
+    Mapping,
     Optional,
     Tuple,
     Type,
@@ -49,7 +50,8 @@ ResponseValue = Union[
     "Response",
     "WerkzeugResponse",
     AnyStr,
-    Dict[str, Any],  # any jsonify-able dict
+    Mapping[str, Any],  # any jsonify-able dict
+    List[Any],  # any jsonify-able list
     AsyncGenerator[AnyStr, None],
     Generator[AnyStr, None, None],
 ]
@@ -93,6 +95,7 @@ ErrorHandlerCallable = Union[
     Callable[[Exception], ResponseReturnValue],
     Callable[[Exception], Awaitable[ResponseReturnValue]],
 ]
+ShellContextProcessorCallable = Callable[[], Dict[str, Any]]
 TeardownCallable = Union[
     Callable[[Optional[BaseException]], None],
     Callable[[Optional[BaseException]], Awaitable[None]],
@@ -105,6 +108,16 @@ TemplateGlobalCallable = Callable[[Any], Any]
 TemplateTestCallable = Callable[[Any], bool]
 URLDefaultCallable = Callable[[str, dict], None]
 URLValuePreprocessorCallable = Callable[[Optional[str], Optional[dict]], None]
+WhileServingCallable = Callable[[], AsyncGenerator[None, None]]
+
+RouteCallable = Union[
+    Callable[..., ResponseReturnValue],
+    Callable[..., Awaitable[ResponseReturnValue]],
+]
+WebsocketCallable = Union[
+    Callable[..., Optional[ResponseReturnValue]],
+    Callable[..., Awaitable[Optional[ResponseReturnValue]]],
+]
 
 
 class ASGIHTTPProtocol(Protocol):
